'use strict';

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactJsonTree = require('@alexkuz/react-json-tree');

var _reactJsonTree2 = _interopRequireDefault(_reactJsonTree);

var _deepMap = require('./deepMap');

var _deepMap2 = _interopRequireDefault(_deepMap);

var _themeable = require('./themeable');

var _themeable2 = _interopRequireDefault(_themeable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function stringify(val) {
  var str = (0, _stringify2.default)(val);

  return str.length > 22 ? str.substr(0, 15) + '…' + str.substr(-5) : str;
}

function prepareDelta(delta) {
  var diffs = [];
  var diffId = -1;
  var data = (0, _deepMap2.default)(delta, function (val) {
    if (Array.isArray(val) || val && val._t === 'a') {
      diffs.push(val);
      diffId++;
      return '<JSON_DIFF_ID>' + diffId;
    }

    return val;
  });

  return { diffs: diffs, data: data };
}

function _valueRenderer(raw, diffs, createTheme) {
  function renderSpan(name, body) {
    return _react2.default.createElement(
      'span',
      (0, _extends3.default)({ key: name }, createTheme('diff', name)),
      body
    );
  }

  if (raw.indexOf('"<JSON_DIFF_ID>') === 0) {
    var _ret = function () {
      var diff = diffs[parseInt(raw.replace(/[^\d]/g, ''), 10)];

      if (Array.isArray(diff)) {
        switch (diff.length) {
          case 1:
            return {
              v: renderSpan('diffAdd', stringify(diff[0]))
            };
          case 2:
            return {
              v: _react2.default.createElement(
                'span',
                null,
                renderSpan('diffUpdateFrom', stringify(diff[0])),
                renderSpan('diffUpdateArrow', ' => '),
                renderSpan('diffUpdateTo', stringify(diff[1]))
              )
            };
          case 3:
            return {
              v: renderSpan('diffRemove', stringify(diff[0]))
            };
        }
      } else if (diff && diff._t === 'a') {
        var _ret2 = function () {
          var keys = (0, _keys2.default)(diff);
          return {
            v: {
              v: _react2.default.createElement(
                'span',
                null,
                '[',
                keys.filter(function (key) {
                  return key !== '_t';
                }).map(function (key, idx) {
                  var last = idx === keys.length - 2;

                  switch (diff[key].length) {
                    case 1:
                      return _react2.default.createElement(
                        'span',
                        null,
                        key + ': ',
                        renderSpan('diffAdd', stringify(diff[key][0])),
                        !last && ', '
                      );
                    case 3:
                      return _react2.default.createElement(
                        'span',
                        null,
                        key.replace('_', '') + ': ',
                        renderSpan('diffRemove', stringify(diff[key][0])),
                        !last && ', '
                      );
                  }
                }),
                ']'
              )
            }
          };
        }();

        if ((typeof _ret2 === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret2)) === "object") return _ret2.v;
      }
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
  }

  return raw;
}

var JSONDiff = function JSONDiff(_ref) {
  var delta = _ref.delta;
  var theme = _ref.theme;
  var defaultTheme = _ref.defaultTheme;
  var props = (0, _objectWithoutProperties3.default)(_ref, ['delta', 'theme', 'defaultTheme']);

  var _prepareDelta = prepareDelta(delta);

  var data = _prepareDelta.data;
  var diffs = _prepareDelta.diffs;

  var createTheme = (0, _themeable2.default)((0, _extends3.default)({}, theme, defaultTheme));

  return _react2.default.createElement(_reactJsonTree2.default, (0, _extends3.default)({}, props, {
    data: data,
    getItemString: function getItemString() {
      return '';
    },
    valueRenderer: function valueRenderer(raw) {
      return _valueRenderer(raw, diffs, createTheme);
    },
    expandAll: true,
    hideRoot: true }));
};

exports.default = JSONDiff;